pub enum Item<'a> {
Literal(&'a str),
OwnedLiteral(Box<str>),
Space(&'a str),
OwnedSpace(Box<str>),
Numeric(Numeric, Pad),
Fixed(Fixed),
Error,
}Expand description
A single formatting item. This is used for both formatting and parsing.
Variants
Literal(&'a str)
A literally printed and parsed text.
OwnedLiteral(Box<str>)
Available on crate feature
alloc only.Same as Literal but with the string owned by the item.
Space(&'a str)
Whitespace. Prints literally but reads zero or more whitespace.
OwnedSpace(Box<str>)
Available on crate feature
alloc only.Same as Space but with the string owned by the item.
Numeric(Numeric, Pad)
Numeric item. Can be optionally padded to the maximal length (if any) when formatting; the parser simply ignores any padded whitespace and zeroes.
Fixed(Fixed)
Fixed-format item.
Error
Issues a formatting error. Used to signal an invalid format string.
Implementations