Executor

Enum Executor 

pub enum Executor<'a, DB>
where DB: Database,
{ Pool(&'a Pool<DB>), Conn(&'a mut <DB as Database>::Connection), }
A unified database executor that can wrap either a connection pool or a single connection.

This allows Premix to remain agnostic about whether it’s executing within a transaction, a shared pool, or a dedicated connection.

Variants



Pool(&'a Pool<DB>)

A shared connection pool.



Conn(&'a mut <DB as Database>::Connection)

A single, mutable database connection.

Implementations

Source

impl<'a, DB> Executor<'a, DB>
where DB: Database,

pub async fn execute<'q, A>( &mut self, query: Query<'q, DB, A>, ) -> Result<<DB as Database>::QueryResult, Error>
where A: IntoArguments<'q, DB> + 'q, DB: SqlDialect, &'c mut <DB as Database>::Connection: for<'c> Executor<'c, Database = DB>,

Executes a SQL query and returns the database result (e.g., number of rows affected).

pub async fn fetch_all<'q, T, A>( &mut self, query: QueryAs<'q, DB, T, A>, ) -> Result<Vec<T>, Error>
where T: for<'r> FromRow<'r, <DB as Database>::Row> + Send + Unpin, A: IntoArguments<'q, DB> + 'q, DB: SqlDialect, &'c mut <DB as Database>::Connection: for<'c> Executor<'c, Database = DB>,

Executes a SQL query and fetches all resulting rows.

pub async fn fetch_optional<'q, T, A>( &mut self, query: QueryAs<'q, DB, T, A>, ) -> Result<Option<T>, Error>
where T: for<'r> FromRow<'r, <DB as Database>::Row> + Send + Unpin, A: IntoArguments<'q, DB> + 'q, DB: SqlDialect, &'c mut <DB as Database>::Connection: for<'c> Executor<'c, Database = DB>,

Executes a SQL query and fetches an optional single row.

pub fn fetch_stream<'q, T, A>( self, query: QueryAs<'q, DB, T, A>, ) -> Pin<Box<dyn Stream<Item = Result<T, Error>> + Send + 'a>>
where 'q: 'a, T: for<'r> FromRow<'r, <DB as Database>::Row> + Send + Unpin + 'a, A: IntoArguments<'q, DB> + 'q, DB: SqlDialect, &'c mut <DB as Database>::Connection: for<'c> Executor<'c, Database = DB>,

Executes a SQL query and returns a stream of resulting rows.