PrometheusHandle

Struct PrometheusHandle 

pub struct PrometheusHandle { /* private fields */ }
Available on crate feature metrics only.
Handle for accessing metrics stored via PrometheusRecorder.

In certain scenarios, it may be necessary to directly handle requests that would otherwise be handled directly by the HTTP listener, or push gateway background task. PrometheusHandle allows rendering a snapshot of the current metrics stored by an installed PrometheusRecorder as a payload conforming to the Prometheus exposition format.

Implementations

Source

impl PrometheusHandle

pub fn render(&self) -> String

Takes a snapshot of the metrics held by the recorder and generates a payload conforming to the Prometheus exposition format.